unit IPCBase;

// Contains the base TIPCBase object which has some useful
// debuging and helper methods

interface

uses SysUtils;

{$IFDEF LINUX}
Const

  LIBC_SUCCESS = 0;

  LIBC_FAILURE = -1;

{$ENDIF}

type
  TIPCBase = class
    private
      FDebug : boolean;
    protected
      procedure DebugMessage(msg : string); virtual;
      procedure ErrorMessage(fn : string); virtual;
    public
      constructor Create;
      property Debug : boolean read FDebug write FDebug;
  end;

  TIPCFilePermission = (fpOwnerRead, fpOwnerWrite, fpOwnerExecute,
                        fpGroupRead, fpGroupWrite, fpGroupExecute,
                        fpOtherRead, fpOtherWrite, fpOtherExecute);


  TIPCFilePermissions = set of TIPCFilePermission;

{$IFDEF LINUX}
function FilePermissionsToBits(perm : TIPCFilePermissions) : integer;
{$ENDIF}

implementation

{$IFDEF WIN32}
uses Windows;
{$ENDIF}

{$IFDEF LINUX}
uses Libc;
{$ENDIF}

{$IFDEF LINUX}
function FilePermissionsToBits(perm : TIPCFilePermissions) : integer;
begin
  Result := 0;
  if fpOwnerRead in perm then
     Result := Result or S_IRUSR;

  if fpOwnerWrite in perm then
     Result := Result or S_IWUSR;

  if fpOwnerExecute in perm then
     Result := Result or S_IXUSR;

  if fpGroupRead in perm then
     Result := Result or S_IRGRP;

  if fpGroupWrite in perm then
     Result := Result or S_IWGRP;

  if fpGroupExecute in perm then
     Result := Result or S_IXGRP;

  if fpOtherRead in perm then
     Result := Result or S_IROTH;

  if fpOtherWrite in perm then
     Result := Result or S_IWOTH;

  if fpOtherExecute in perm then
     Result := Result or S_IXOTH;

end;
{$ENDIF}


{ TIPCBase }

constructor TIPCBase.Create;
begin
  inherited;
  FDebug := false;
end;

procedure TIPCBase.DebugMessage(msg: string);
begin
  if FDebug then
     writeln(msg);
end;

procedure TIPCBase.ErrorMessage(fn : string);
begin
  // Note:
  //  GetLastError has been implemented in Linux to return the errno global variable.
  raise Exception.CreateFmt('%s failed. Error is: %d ',[fn,GetLastError]);
end;

end.
