unit Semaphore;

interface

uses IPCBase,Libc;

type

  // Important Point: Semaphores can be used like a mutex
  // but they can be unlocked by another thread
  TPosixSemaphore = class (TIPCBase)
  private
    FSem    : TSemaphore;
    function GetCurrentValue: integer;
  protected
  public
    Constructor Create(initialValue : integer = 0);
    Destructor Destroy; override;
    procedure Wait;
    function  TryWait : boolean;
    procedure Post;
  published
    property CurrentValue : integer             read GetCurrentValue;
  end;

implementation

{ TPosixSemaphore }

constructor TPosixSemaphore.Create(initialValue : integer = 0);
var
  ret : integer;

begin
  inherited Create;

  // memory based semaphores use sem_init
  // NOTE: Currently LINUX does not support process-shared semaphores
  ret := sem_init(FSem, false, initialValue);
  if ret <> LIBC_SUCCESS then
    ErrorMessage('sem_init');
end;

destructor TPosixSemaphore.Destroy;
var
  ret : integer;

begin
  ret := sem_destroy( FSem );
  if ret <> LIBC_SUCCESS then
    ErrorMessage('sem_destroy');
  inherited;
end;

function TPosixSemaphore.GetCurrentValue: integer;
var
  ret : integer;

begin
  ret := sem_getvalue( FSem,Result);
  if ret <> LIBC_SUCCESS then
    ErrorMessage('sem_getvalue');
end;

procedure TPosixSemaphore.Post;
var
  ret : integer;

begin
  ret := sem_post(FSem);
  if ret <> LIBC_SUCCESS then
    ErrorMessage('sem_post');
end;

function TPosixSemaphore.TryWait: boolean;
var
  ret : integer;

begin
  ret := sem_trywait(FSem);
  Result := (ret = LIBC_SUCCESS);
end;

procedure TPosixSemaphore.Wait;
var
  ret : integer;

begin
  ret := sem_wait(FSem);
  if ret <> LIBC_SUCCESS then
    ErrorMessage('sem_wait');
end;

end.

